/* Generated by dbus-binding-tool; do not edit! */

/* This file is generated by glib-genmarshal, do not modify it. This code is licensed under the same license as the containing project. Note that it links to GLib, so must comply with the LGPL linking clauses. */
#include <glib-object.h>

#ifdef G_ENABLE_DEBUG
#define g_marshal_value_peek_boolean(v)  g_value_get_boolean (v)
#define g_marshal_value_peek_char(v)     g_value_get_schar (v)
#define g_marshal_value_peek_uchar(v)    g_value_get_uchar (v)
#define g_marshal_value_peek_int(v)      g_value_get_int (v)
#define g_marshal_value_peek_uint(v)     g_value_get_uint (v)
#define g_marshal_value_peek_long(v)     g_value_get_long (v)
#define g_marshal_value_peek_ulong(v)    g_value_get_ulong (v)
#define g_marshal_value_peek_int64(v)    g_value_get_int64 (v)
#define g_marshal_value_peek_uint64(v)   g_value_get_uint64 (v)
#define g_marshal_value_peek_enum(v)     g_value_get_enum (v)
#define g_marshal_value_peek_flags(v)    g_value_get_flags (v)
#define g_marshal_value_peek_float(v)    g_value_get_float (v)
#define g_marshal_value_peek_double(v)   g_value_get_double (v)
#define g_marshal_value_peek_string(v)   (char*) g_value_get_string (v)
#define g_marshal_value_peek_param(v)    g_value_get_param (v)
#define g_marshal_value_peek_boxed(v)    g_value_get_boxed (v)
#define g_marshal_value_peek_pointer(v)  g_value_get_pointer (v)
#define g_marshal_value_peek_object(v)   g_value_get_object (v)
#define g_marshal_value_peek_variant(v)  g_value_get_variant (v)
#else /* !G_ENABLE_DEBUG */
/* WARNING: This code accesses GValues directly, which is UNSUPPORTED API.
 *          Do not access GValues directly in your code. Instead, use the
 *          g_value_get_*() functions
 */
#define g_marshal_value_peek_boolean(v)  (v)->data[0].v_int
#define g_marshal_value_peek_char(v)     (v)->data[0].v_int
#define g_marshal_value_peek_uchar(v)    (v)->data[0].v_uint
#define g_marshal_value_peek_int(v)      (v)->data[0].v_int
#define g_marshal_value_peek_uint(v)     (v)->data[0].v_uint
#define g_marshal_value_peek_long(v)     (v)->data[0].v_long
#define g_marshal_value_peek_ulong(v)    (v)->data[0].v_ulong
#define g_marshal_value_peek_int64(v)    (v)->data[0].v_int64
#define g_marshal_value_peek_uint64(v)   (v)->data[0].v_uint64
#define g_marshal_value_peek_enum(v)     (v)->data[0].v_long
#define g_marshal_value_peek_flags(v)    (v)->data[0].v_ulong
#define g_marshal_value_peek_float(v)    (v)->data[0].v_float
#define g_marshal_value_peek_double(v)   (v)->data[0].v_double
#define g_marshal_value_peek_string(v)   (v)->data[0].v_pointer
#define g_marshal_value_peek_param(v)    (v)->data[0].v_pointer
#define g_marshal_value_peek_boxed(v)    (v)->data[0].v_pointer
#define g_marshal_value_peek_pointer(v)  (v)->data[0].v_pointer
#define g_marshal_value_peek_object(v)   (v)->data[0].v_pointer
#define g_marshal_value_peek_variant(v)  (v)->data[0].v_pointer
#endif /* !G_ENABLE_DEBUG */

/* Prototype for -Wmissing-prototypes */
G_BEGIN_DECLS
extern
void dbus_glib_marshal_notification_manager_BOOLEAN__UINT_POINTER (GClosure     *closure,
                                                                   GValue       *return_value,
                                                                   guint         n_param_values,
                                                                   const GValue *param_values,
                                                                   gpointer      invocation_hint,
                                                                   gpointer      marshal_data);
G_END_DECLS
void
dbus_glib_marshal_notification_manager_BOOLEAN__UINT_POINTER (GClosure     *closure,
                                                              GValue       *return_value,
                                                              guint         n_param_values,
                                                              const GValue *param_values,
                                                              gpointer      invocation_hint G_GNUC_UNUSED,
                                                              gpointer      marshal_data)
{
  typedef gboolean (*GMarshalFunc_BOOLEAN__UINT_POINTER) (gpointer data1,
                                                          guint arg1,
                                                          gpointer arg2,
                                                          gpointer data2);
  GCClosure *cc = (GCClosure *) closure;
  gpointer data1, data2;
  GMarshalFunc_BOOLEAN__UINT_POINTER callback;
  gboolean v_return;

  g_return_if_fail (return_value != NULL);
  g_return_if_fail (n_param_values == 3);

  if (G_CCLOSURE_SWAP_DATA (closure))
    {
      data1 = closure->data;
      data2 = g_value_peek_pointer (param_values + 0);
    }
  else
    {
      data1 = g_value_peek_pointer (param_values + 0);
      data2 = closure->data;
    }
  callback = (GMarshalFunc_BOOLEAN__UINT_POINTER) (marshal_data ? marshal_data : cc->callback);

  v_return = callback (data1,
                       g_marshal_value_peek_uint (param_values + 1),
                       g_marshal_value_peek_pointer (param_values + 2),
                       data2);

  g_value_set_boolean (return_value, v_return);
}

/* Prototype for -Wmissing-prototypes */
G_BEGIN_DECLS
extern
void dbus_glib_marshal_notification_manager_BOOLEAN__POINTER_POINTER (GClosure     *closure,
                                                                      GValue       *return_value,
                                                                      guint         n_param_values,
                                                                      const GValue *param_values,
                                                                      gpointer      invocation_hint,
                                                                      gpointer      marshal_data);
G_END_DECLS
void
dbus_glib_marshal_notification_manager_BOOLEAN__POINTER_POINTER (GClosure     *closure,
                                                                 GValue       *return_value,
                                                                 guint         n_param_values,
                                                                 const GValue *param_values,
                                                                 gpointer      invocation_hint G_GNUC_UNUSED,
                                                                 gpointer      marshal_data)
{
  typedef gboolean (*GMarshalFunc_BOOLEAN__POINTER_POINTER) (gpointer data1,
                                                             gpointer arg1,
                                                             gpointer arg2,
                                                             gpointer data2);
  GCClosure *cc = (GCClosure *) closure;
  gpointer data1, data2;
  GMarshalFunc_BOOLEAN__POINTER_POINTER callback;
  gboolean v_return;

  g_return_if_fail (return_value != NULL);
  g_return_if_fail (n_param_values == 3);

  if (G_CCLOSURE_SWAP_DATA (closure))
    {
      data1 = closure->data;
      data2 = g_value_peek_pointer (param_values + 0);
    }
  else
    {
      data1 = g_value_peek_pointer (param_values + 0);
      data2 = closure->data;
    }
  callback = (GMarshalFunc_BOOLEAN__POINTER_POINTER) (marshal_data ? marshal_data : cc->callback);

  v_return = callback (data1,
                       g_marshal_value_peek_pointer (param_values + 1),
                       g_marshal_value_peek_pointer (param_values + 2),
                       data2);

  g_value_set_boolean (return_value, v_return);
}

/* Prototype for -Wmissing-prototypes */
G_BEGIN_DECLS
extern
void dbus_glib_marshal_notification_manager_BOOLEAN__POINTER_POINTER_POINTER_POINTER_POINTER (GClosure     *closure,
                                                                                              GValue       *return_value,
                                                                                              guint         n_param_values,
                                                                                              const GValue *param_values,
                                                                                              gpointer      invocation_hint,
                                                                                              gpointer      marshal_data);
G_END_DECLS
void
dbus_glib_marshal_notification_manager_BOOLEAN__POINTER_POINTER_POINTER_POINTER_POINTER (GClosure     *closure,
                                                                                         GValue       *return_value,
                                                                                         guint         n_param_values,
                                                                                         const GValue *param_values,
                                                                                         gpointer      invocation_hint G_GNUC_UNUSED,
                                                                                         gpointer      marshal_data)
{
  typedef gboolean (*GMarshalFunc_BOOLEAN__POINTER_POINTER_POINTER_POINTER_POINTER) (gpointer data1,
                                                                                     gpointer arg1,
                                                                                     gpointer arg2,
                                                                                     gpointer arg3,
                                                                                     gpointer arg4,
                                                                                     gpointer arg5,
                                                                                     gpointer data2);
  GCClosure *cc = (GCClosure *) closure;
  gpointer data1, data2;
  GMarshalFunc_BOOLEAN__POINTER_POINTER_POINTER_POINTER_POINTER callback;
  gboolean v_return;

  g_return_if_fail (return_value != NULL);
  g_return_if_fail (n_param_values == 6);

  if (G_CCLOSURE_SWAP_DATA (closure))
    {
      data1 = closure->data;
      data2 = g_value_peek_pointer (param_values + 0);
    }
  else
    {
      data1 = g_value_peek_pointer (param_values + 0);
      data2 = closure->data;
    }
  callback = (GMarshalFunc_BOOLEAN__POINTER_POINTER_POINTER_POINTER_POINTER) (marshal_data ? marshal_data : cc->callback);

  v_return = callback (data1,
                       g_marshal_value_peek_pointer (param_values + 1),
                       g_marshal_value_peek_pointer (param_values + 2),
                       g_marshal_value_peek_pointer (param_values + 3),
                       g_marshal_value_peek_pointer (param_values + 4),
                       g_marshal_value_peek_pointer (param_values + 5),
                       data2);

  g_value_set_boolean (return_value, v_return);
}

/* Prototype for -Wmissing-prototypes */
G_BEGIN_DECLS
extern
void dbus_glib_marshal_notification_manager_BOOLEAN__STRING_UINT_STRING_STRING_STRING_BOXED_BOXED_INT_POINTER_POINTER (GClosure     *closure,
                                                                                                                       GValue       *return_value,
                                                                                                                       guint         n_param_values,
                                                                                                                       const GValue *param_values,
                                                                                                                       gpointer      invocation_hint,
                                                                                                                       gpointer      marshal_data);
G_END_DECLS
void
dbus_glib_marshal_notification_manager_BOOLEAN__STRING_UINT_STRING_STRING_STRING_BOXED_BOXED_INT_POINTER_POINTER (GClosure     *closure,
                                                                                                                  GValue       *return_value,
                                                                                                                  guint         n_param_values,
                                                                                                                  const GValue *param_values,
                                                                                                                  gpointer      invocation_hint G_GNUC_UNUSED,
                                                                                                                  gpointer      marshal_data)
{
  typedef gboolean (*GMarshalFunc_BOOLEAN__STRING_UINT_STRING_STRING_STRING_BOXED_BOXED_INT_POINTER_POINTER) (gpointer data1,
                                                                                                              gpointer arg1,
                                                                                                              guint arg2,
                                                                                                              gpointer arg3,
                                                                                                              gpointer arg4,
                                                                                                              gpointer arg5,
                                                                                                              gpointer arg6,
                                                                                                              gpointer arg7,
                                                                                                              gint arg8,
                                                                                                              gpointer arg9,
                                                                                                              gpointer arg10,
                                                                                                              gpointer data2);
  GCClosure *cc = (GCClosure *) closure;
  gpointer data1, data2;
  GMarshalFunc_BOOLEAN__STRING_UINT_STRING_STRING_STRING_BOXED_BOXED_INT_POINTER_POINTER callback;
  gboolean v_return;

  g_return_if_fail (return_value != NULL);
  g_return_if_fail (n_param_values == 11);

  if (G_CCLOSURE_SWAP_DATA (closure))
    {
      data1 = closure->data;
      data2 = g_value_peek_pointer (param_values + 0);
    }
  else
    {
      data1 = g_value_peek_pointer (param_values + 0);
      data2 = closure->data;
    }
  callback = (GMarshalFunc_BOOLEAN__STRING_UINT_STRING_STRING_STRING_BOXED_BOXED_INT_POINTER_POINTER) (marshal_data ? marshal_data : cc->callback);

  v_return = callback (data1,
                       g_marshal_value_peek_string (param_values + 1),
                       g_marshal_value_peek_uint (param_values + 2),
                       g_marshal_value_peek_string (param_values + 3),
                       g_marshal_value_peek_string (param_values + 4),
                       g_marshal_value_peek_string (param_values + 5),
                       g_marshal_value_peek_boxed (param_values + 6),
                       g_marshal_value_peek_boxed (param_values + 7),
                       g_marshal_value_peek_int (param_values + 8),
                       g_marshal_value_peek_pointer (param_values + 9),
                       g_marshal_value_peek_pointer (param_values + 10),
                       data2);

  g_value_set_boolean (return_value, v_return);
}

#include <dbus/dbus-glib.h>
static const DBusGMethodInfo dbus_glib_notification_manager_methods[] = {
  { (GCallback) notification_manager_notify, dbus_glib_marshal_notification_manager_BOOLEAN__STRING_UINT_STRING_STRING_STRING_BOXED_BOXED_INT_POINTER_POINTER, 0 },
  { (GCallback) notification_manager_close_notification, dbus_glib_marshal_notification_manager_BOOLEAN__UINT_POINTER, 147 },
  { (GCallback) notification_manager_get_capabilities, dbus_glib_marshal_notification_manager_BOOLEAN__POINTER_POINTER, 205 },
  { (GCallback) notification_manager_get_server_information, dbus_glib_marshal_notification_manager_BOOLEAN__POINTER_POINTER_POINTER_POINTER_POINTER, 268 },
};

const DBusGObjectInfo dbus_glib_notification_manager_object_info = {  1,
  dbus_glib_notification_manager_methods,
  4,
"org.freedesktop.Notifications\0Notify\0S\0app_name\0I\0s\0id\0I\0u\0icon\0I\0s\0summary\0I\0s\0body\0I\0s\0actions\0I\0as\0hints\0I\0a{sv}\0timeout\0I\0i\0return_id\0O\0F\0N\0u\0\0org.freedesktop.Notifications\0CloseNotification\0S\0id\0I\0u\0\0org.freedesktop.Notifications\0GetCapabilities\0S\0caps\0O\0F\0N\0as\0\0org.freedesktop.Notifications\0GetServerInformation\0S\0name\0O\0F\0N\0s\0vendor\0O\0F\0N\0s\0version\0O\0F\0N\0s\0spec_version\0O\0F\0N\0s\0\0\0",
"org.freedesktop.Notifications\0NotificationClosed\0\0",
"\0"
};

